@@ -4,7 +4,7 @@ Attean - A Semantic Web Framework
 
 =head1 VERSION
 
-This document describes Attean version 0.003
+This document describes Attean version 0.004
 
 =head1 SYNOPSIS
 
@@ -41,7 +41,7 @@ Semantic Web (RDF and SPARQL) data.
 package Attean {
 	use v5.14;
 	use warnings;
-	our $VERSION	= '0.003';
+	our $VERSION	= '0.004';
 	use Attean::API;
 	
 	use Attean::Blank;
@@ -54,11 +54,14 @@ package Attean {
 	use Attean::Result;
 	
 	use Attean::QuadModel;
+	use Attean::TripleModel;
 	use Attean::BindingEqualityTest;
 	
 	use Attean::CodeIterator;
 	use Attean::ListIterator;
 	use Attean::IteratorSequence;
+
+	use Attean::IDPQueryPlanner;
 	
 	use Attean::TermMap;
 	
@@ -166,6 +169,7 @@ returns undef.
 		if (my $method = $method{ $type }) {
 			my $value	= shift;
 			$value	=~ s/^.*[.]// if ($type eq 'filename');
+			$value	=~ s/;.*$// if ($type eq 'media_type');
 			foreach my $p ($self->parsers()) {
 				if (can_load( modules => { $p => 0 })) {
 					next unless ($p->does($role));
