@@ -7,7 +7,7 @@ Attean::API::Binding - Name to term bindings
 
 =head1 VERSION
 
-This document describes Attean::API::Binding version 0.003
+This document describes Attean::API::Binding version 0.004
 
 =head1 DESCRIPTION
 
@@ -71,7 +71,7 @@ otherwise.
 
 use Type::Tiny::Role;
 
-package Attean::API::Binding 0.001 {
+package Attean::API::Binding 0.004 {
 	use Moo::Role;
 	use List::MoreUtils qw(zip);
 	use namespace::clean;
@@ -107,13 +107,19 @@ package Attean::API::Binding 0.001 {
 		return scalar(@blanks);
 	}
 	
+	sub values_consuming_role {
+		my $self	= shift;
+		my $role	= shift;
+		return grep { $_->does($role) } $self->values;
+	}
+	
 	sub tree_attributes {
 		my $self	= shift;
 		return $self->variables;
 	};
 }
 
-package Attean::API::TripleOrQuadPattern 0.001 {
+package Attean::API::TripleOrQuadPattern 0.004 {
 	use Moo::Role;
 	sub apply_map {
 		my $self	= shift;
@@ -124,12 +130,12 @@ package Attean::API::TripleOrQuadPattern 0.001 {
 	}
 }
 
-package Attean::API::TripleOrQuad 0.001 {
+package Attean::API::TripleOrQuad 0.004 {
 	use Moo::Role;
 	with 'Attean::API::TripleOrQuadPattern';
 }
 
-package Attean::API::TriplePattern 0.001 {
+package Attean::API::TriplePattern 0.004 {
 	use Moo::Role;
 	use List::MoreUtils qw(zip);
 	use namespace::clean;
@@ -150,7 +156,12 @@ package Attean::API::TriplePattern 0.001 {
 		my @values	= ($self->values, $graph);
 		return Attean::QuadPattern->new(zip @keys, @values);
 	}
-
+	
+	sub as_sparql {
+		my $self	= shift;
+		return join(' ', map { $_->ntriples_string } $self->values) . " .\n";
+	}
+	
 	requires 'subject';
 	requires 'predicate';
 	requires 'object';
@@ -158,7 +169,7 @@ package Attean::API::TriplePattern 0.001 {
 	with 'Attean::API::TripleOrQuadPattern', 'Attean::API::Binding';
 }
 
-package Attean::API::Triple 0.001 {
+package Attean::API::Triple 0.004 {
 	use Moo::Role;
 	
 	if ($ENV{ATTEAN_TYPECHECK}) {
@@ -193,7 +204,7 @@ package Attean::API::Triple 0.001 {
 	with 'Attean::API::TriplePattern', 'Attean::API::TripleOrQuad', 'Attean::API::Binding';
 }
 
-package Attean::API::QuadPattern 0.001 {
+package Attean::API::QuadPattern 0.004 {
 	use Moo::Role;
 	
 	sub variables { return qw(subject predicate object graph) }
@@ -212,7 +223,7 @@ package Attean::API::QuadPattern 0.001 {
 	with 'Attean::API::TripleOrQuadPattern', 'Attean::API::Binding';
 }
 
-package Attean::API::Quad 0.001 {
+package Attean::API::Quad 0.004 {
 	use Moo::Role;
 	
 	if ($ENV{ATTEAN_TYPECHECK}) {
@@ -239,7 +250,7 @@ package Attean::API::Quad 0.001 {
 }
 
 
-package Attean::API::Result 0.001 {
+package Attean::API::Result 0.004 {
 	use Moo::Role;
 	use Scalar::Util qw(refaddr);
 	use namespace::clean;
