@@ -7,7 +7,7 @@ Attean::API::Parser - Parser role
 
 =head1 VERSION
 
-This document describes Attean::API::Parser version 0.003
+This document describes Attean::API::Parser version 0.004
 
 =head1 DESCRIPTION
 
@@ -52,7 +52,7 @@ parsing.
 
 use Type::Tiny::Role;
 
-package Attean::API::Parser 0.001 {
+package Attean::API::Parser 0.004 {
 	use Moo::Role;
 	use Types::Standard qw(CodeRef);
 	use namespace::clean;
@@ -65,7 +65,7 @@ package Attean::API::Parser 0.001 {
 	requires 'file_extensions'; # => (is => 'ro', isa => 'ArrayRef[Str]', init_arg => undef);
 }
 
-package Attean::API::AbbreviatingParser 0.001 {
+package Attean::API::AbbreviatingParser 0.004 {
 	use Moo::Role;
 	use Types::Standard qw(InstanceOf Maybe);
 	use URI::NamespaceMap;
@@ -77,7 +77,7 @@ package Attean::API::AbbreviatingParser 0.001 {
 	has 'namespaces'	=> (is => 'ro', isa => Maybe[InstanceOf['URI::NamespaceMap']]);
 }
 
-package Attean::API::PushParser 0.001 {
+package Attean::API::PushParser 0.004 {
 	use Moo::Role;
 	with 'Attean::API::Parser';
 
@@ -87,13 +87,37 @@ package Attean::API::PushParser 0.001 {
 	sub parse_iter_from_io {
 		my $self	= shift;
 		my @values	= $self->parse_list_from_io(@_);
-		return Attean::ListIterator->new( values => \@values, item_type => $self->handled_type->role, );
+		if ($self->does('Attean::API::ResultParser') or $self->does('Attean::API::ResultOrTermParser')) {
+			my %vars;
+			foreach my $r (@values) {
+				if ($r->does('Attean::API::Result')) {
+					foreach my $v ($r->variables) {
+						$vars{$v}++;
+					}
+				}
+			}
+			return Attean::ListIterator->new( variables => [keys %vars], values => \@values, item_type => $self->handled_type->role, );
+		} else {
+			return Attean::ListIterator->new( values => \@values, item_type => $self->handled_type->role, );
+		}
 	}
 	
 	sub parse_iter_from_bytes {
 		my $self	= shift;
 		my @values	= $self->parse_list_from_bytes(@_);
-		return Attean::ListIterator->new( values => \@values, item_type => $self->handled_type->role, );
+		if ($self->does('Attean::API::ResultParser') or $self->does('Attean::API::ResultOrTermParser')) {
+			my %vars;
+			foreach my $r (@values) {
+				if ($r->does('Attean::API::Result')) {
+					foreach my $v ($r->variables) {
+						$vars{$v}++;
+					}
+				}
+			}
+			return Attean::ListIterator->new( variables => [keys %vars], values => \@values, item_type => $self->handled_type->role, );
+		} else {
+			return Attean::ListIterator->new( values => \@values, item_type => $self->handled_type->role, );
+		}
 	}
 	
 	sub parse_list_from_io {
@@ -117,7 +141,7 @@ package Attean::API::PushParser 0.001 {
 	}
 }
 
-package Attean::API::PullParser 0.001 {
+package Attean::API::PullParser 0.004 {
 	use Moo::Role;
 	with 'Attean::API::Parser';
 	
@@ -159,7 +183,7 @@ package Attean::API::PullParser 0.001 {
 	}
 }
 
-package Attean::API::AtOnceParser 0.001 {
+package Attean::API::AtOnceParser 0.004 {
 	use Moo::Role;
 	with 'Attean::API::Parser';
 	
@@ -185,18 +209,42 @@ package Attean::API::AtOnceParser 0.001 {
 	sub parse_iter_from_io {
 		my $self	= shift;
 		my @values	= $self->parse_list_from_io(@_);
-		return Attean::ListIterator->new( values => \@values, item_type => $self->handled_type->role, );
+		if ($self->does('Attean::API::ResultParser') or $self->does('Attean::API::ResultOrTermParser')) {
+			my %vars;
+			foreach my $r (@values) {
+				if ($r->does('Attean::API::Result')) {
+					foreach my $v ($r->variables) {
+						$vars{$v}++;
+					}
+				}
+			}
+			return Attean::ListIterator->new( variables => [keys %vars], values => \@values, item_type => $self->handled_type->role, );
+		} else {
+			return Attean::ListIterator->new( values => \@values, item_type => $self->handled_type->role, );
+		}
 	}
 	
 	sub parse_iter_from_bytes {
 		my $self	= shift;
 		my @values	= $self->parse_list_from_bytes(@_);
-		return Attean::ListIterator->new( values => \@values, item_type => $self->handled_type->role, );
+		if ($self->does('Attean::API::ResultParser') or $self->does('Attean::API::ResultOrTermParser')) {
+			my %vars;
+			foreach my $r (@values) {
+				if ($r->does('Attean::API::Result')) {
+					foreach my $v ($r->variables) {
+						$vars{$v}++;
+					}
+				}
+			}
+			return Attean::ListIterator->new( variables => [keys %vars], values => \@values, item_type => $self->handled_type->role, );
+		} else {
+			return Attean::ListIterator->new( values => \@values, item_type => $self->handled_type->role, );
+		}
 	}
 	
 }
 
-package Attean::API::TermParser 0.001 {
+package Attean::API::TermParser 0.004 {
 	# Parser returns objects that conform to Attean::API::Term
 	use Moo::Role;
 	with 'Attean::API::Parser';
@@ -206,7 +254,7 @@ package Attean::API::TermParser 0.001 {
 	}
 }
 
-package Attean::API::TripleParser 0.001 {
+package Attean::API::TripleParser 0.004 {
 	# Parser returns objects that conform to Attean::API::Triple
 	use Moo::Role;
 	with 'Attean::API::Parser';
@@ -216,7 +264,7 @@ package Attean::API::TripleParser 0.001 {
 	}
 }
 
-package Attean::API::QuadParser 0.001 {
+package Attean::API::QuadParser 0.004 {
 	# Parser returns objects that conform to Attean::API::Quad
 	use Moo::Role;
 	with 'Attean::API::Parser';
@@ -226,7 +274,7 @@ package Attean::API::QuadParser 0.001 {
 	}
 }
 
-package Attean::API::MixedStatementParser 0.001 {
+package Attean::API::MixedStatementParser 0.004 {
 	# Parser returns objects that conform to either Attean::API::Triple or Attean::API::Quad
 	use Moo::Role;
 	with 'Attean::API::Parser';
@@ -236,7 +284,7 @@ package Attean::API::MixedStatementParser 0.001 {
 	}
 }
 
-package Attean::API::ResultOrTermParser 0.001 {
+package Attean::API::ResultOrTermParser 0.004 {
 	# Parser returns objects that conform to either Attean::API::Result or Attean::API::Term
 	use Moo::Role;
 	with 'Attean::API::Parser';
@@ -246,7 +294,7 @@ package Attean::API::ResultOrTermParser 0.001 {
 	}
 }
 
-package Attean::API::ResultParser 0.001 {
+package Attean::API::ResultParser 0.004 {
 	# Parser returns objects that conform to Attean::API::Result
 	use Moo::Role;
 	with 'Attean::API::Parser';
