@@ -7,7 +7,7 @@ Attean::Expression - SPARQL Expressions
 
 =head1 VERSION
 
-This document describes Attean::Expression version 0.003
+This document describes Attean::Expression version 0.004
 
 =head1 SYNOPSIS
 
@@ -40,7 +40,7 @@ use Attean::API::Expression;
 
 =cut
 
-package Attean::ValueExpression 0.001 {
+package Attean::ValueExpression 0.004 {
 	use Moo;
 	use Types::Standard qw(ConsumerOf);
 	use namespace::clean;
@@ -62,13 +62,26 @@ package Attean::ValueExpression 0.001 {
 		}
 		return $str;
 	}
+	
+	sub in_scope_variables {
+		my $self	= shift;
+		if ($self->value->does('Attean::API::Variable')) {
+			return $self->value->value;
+		}
+		return;
+	}
+	
+	sub as_sparql {
+		my $self	= shift;
+		return $self->value->as_sparql;
+	}
 }
 
 =item * L<Attean::UnaryExpression>
 
 =cut
 
-package Attean::UnaryExpression 0.001 {
+package Attean::UnaryExpression 0.004 {
 	use Moo;
 	use Types::Standard qw(Enum);
 	use namespace::clean;
@@ -95,7 +108,7 @@ package Attean::UnaryExpression 0.001 {
 
 =cut
 
-package Attean::BinaryExpression 0.001 {
+package Attean::BinaryExpression 0.004 {
 	use Moo;
 	use Types::Standard qw(Enum);
 	use namespace::clean;
@@ -113,7 +126,7 @@ package Attean::BinaryExpression 0.001 {
 
 =cut
 
-package Attean::FunctionExpression 0.001 {
+package Attean::FunctionExpression 0.004 {
 	use Moo;
 	use Types::Standard qw(Enum ConsumerOf HashRef);
 	use Types::Common::String qw(UpperCaseStr);
@@ -134,9 +147,22 @@ package Attean::FunctionExpression 0.001 {
 	has 'operator'		=> (is => 'ro', isa => UpperCaseStr, coerce => UpperCaseStr->coercion, required => 1);
 	has 'base'			=> (is => 'rw', isa => ConsumerOf['Attean::IRI'], predicate => 'has_base');
 	with 'Attean::API::NaryExpression';
+	sub as_sparql {
+		my $self	= shift;
+		my $op		= $self->operator;
+		my @args	= @{ $self->children };
+		if ($op =~ /^(NOT)?IN$/) {
+			$op		=~ s/NOT/NOT /;
+			my ($t, @values)	= map { $_->as_sparql } @args;
+			return sprintf("%s %s (%s)", $t, $op, join(', ', @values));
+		} else {
+			my @values	= map { $_->as_sparql } @args;
+			return sprintf("%s(%s)", $op, join(', ', @values));
+		}
+	}
 }
 
-package Attean::AggregateExpression 0.001 {
+package Attean::AggregateExpression 0.004 {
 	use Moo;
 	use Types::Standard qw(Bool Enum Str HashRef ConsumerOf);
 	use Types::Common::String qw(UpperCaseStr);
@@ -160,7 +186,7 @@ package Attean::AggregateExpression 0.001 {
 	with 'Attean::API::AggregateExpression';
 }
 
-package Attean::CastExpression 0.001 {
+package Attean::CastExpression 0.004 {
 	use Moo;
 	use Types::Standard qw(Enum ConsumerOf);
 	use namespace::clean;
@@ -179,7 +205,7 @@ package Attean::CastExpression 0.001 {
 	with 'Attean::API::UnaryExpression', 'Attean::API::Expression', 'Attean::API::UnaryQueryTree';
 }
 
-package Attean::ExistsExpression 0.001 {
+package Attean::ExistsExpression 0.004 {
 	use Moo;
 	use Types::Standard qw(ConsumerOf);
 	use namespace::clean;
@@ -194,8 +220,20 @@ package Attean::ExistsExpression 0.001 {
 	sub as_string {
 		my $self	= shift;
 		# TODO: implement as_string for EXISTS patterns
+		warn "TODO: Attean::ExistsExpression->as_string";
 		return "EXISTS { ... }";
 	}
+	sub as_sparql {
+		my $self	= shift;
+		my %args	= @_;
+		my $level	= $args{level} // 0;
+		my $sp		= $args{indent} // '    ';
+		my $indent	= $sp x $level;
+
+		# TODO: implement as_string for EXISTS patterns
+		warn "TODO: Attean::ExistsExpression->as_string";
+		return "EXISTS " . $self->pattern->as_sparql( level => $level+1, indent => $sp );
+	}
 }
 
 
