@@ -4,7 +4,7 @@ AtteanX::Serializer::SPARQLCSV - SPARQL Results CSV Serializer
 
 =head1 VERSION
 
-This document describes AtteanX::Serializer::SPARQLCSV version 0.034
+This document describes AtteanX::Serializer::SPARQLCSV version 0.004
 
 =head1 SYNOPSIS
 
@@ -16,16 +16,6 @@ This document describes AtteanX::Serializer::SPARQLCSV version 0.034
 
 ...
 
-=head1 ATTRIBUTES
-
-=over 4
-
-=item C<< canonical_media_type >>
-
-=item C<< file_extensions >>
-
-=back
-
 =head1 METHODS
 
 =over 4
@@ -35,13 +25,13 @@ This document describes AtteanX::Serializer::SPARQLCSV version 0.034
 use v5.14;
 use warnings;
 
-package AtteanX::Serializer::SPARQLCSV 0.034 {
+package AtteanX::Serializer::SPARQLCSV 0.004 {
 	use Moo;
 	use Types::Standard qw(Str ArrayRef);
 	use Encode qw(encode);
 	use Scalar::Util qw(blessed);
 	use Attean::ListIterator;
-	use List::Util qw(any);
+	use List::MoreUtils qw(any);
 	use Text::CSV;
 	use namespace::clean;
 
@@ -56,14 +46,6 @@ Returns a list of media types that identify the format produced by this serializ
 	sub media_types {
 		return [qw(text/csv)];
 	}
-
-=item C<< file_extensions >>
-
-Returns a list of file extensions associated with the serialized format.
-
-=cut
-
-	sub file_extensions { return [qw(csv)] };
 	
 =item C<< serialize_iter_to_io( $fh, $iterator ) >>
 
@@ -76,16 +58,16 @@ L<IO::Handle> object C<< $fh >>.
 		my $self	= shift;
 		my $io		= shift;
 		my $iter	= shift;
+		my $first	= 1;
 		my $csv = Text::CSV->new ( { binary => 1 } );
-
-		my @vars	= @{ $iter->variables };
-		$csv->print($io, \@vars);
-		print $io "\n";
-		
 		while (my $t = $iter->next()) {
+			if ($first) {
+				$csv->print($io, [$t->variables]);
+				print $io "\n";
+				$first	= 0;
+			}
 			my @strings;
-			foreach my $var (@vars) {
-				my $term	= $t->value($var);
+			foreach my $term ($t->values) {
 				if (blessed($term)) {
 					if ($term->does('Attean::API::Blank')) {
 						push(@strings, $term->ntriples_string);
@@ -102,7 +84,7 @@ L<IO::Handle> object C<< $fh >>.
 		return;
 	}
 	
-=item C<< serialize_iter_to_bytes( $iterator ) >>
+=item C<< serialize_iter_to_bytes( $fh, $iterator ) >>
 
 Serializes the L<Attean::API::Binding> objects from C<< $iterator >>
 and returns the serialization as a UTF-8 encoded byte string.
@@ -113,16 +95,17 @@ and returns the serialization as a UTF-8 encoded byte string.
 		my $self	= shift;
 		my $iter	= shift;
 		my $data	= encode('UTF-8', '');
+		my $first	= 1;
 		my $csv = Text::CSV->new ( { binary => 1 } );
-
-		my @vars	= @{ $iter->variables };
-		$csv->combine(map { encode('UTF-8', $_) } @vars);
-		$data		.= $csv->string . "\n";
-
 		while (my $t = $iter->next()) {
+			if ($first) {
+				if ($csv->combine(map { encode('UTF-8', $_) } $t->variables)) {
+					$data	.= $csv->string . "\n";
+				}
+				$first	= 0;
+			}
 			my @strings;
-			foreach my $var (@vars) {
-				my $term	= $t->value($var);
+			foreach my $term ($t->values) {
 				if (blessed($term)) {
 					if ($term->does('Attean::API::Blank')) {
 						push(@strings, $term->ntriples_string);
@@ -160,7 +143,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2014--2022 Gregory Todd Williams. This
+Copyright (c) 2006-2012 Gregory Todd Williams. This
 program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
