@@ -4,7 +4,7 @@ AtteanX::Serializer::SPARQLXML - SPARQL Results XML Serializer
 
 =head1 VERSION
 
-This document describes AtteanX::Serializer::SPARQLXML version 0.034
+This document describes AtteanX::Serializer::SPARQLXML version 0.004
 
 =head1 SYNOPSIS
 
@@ -16,16 +16,6 @@ This document describes AtteanX::Serializer::SPARQLXML version 0.034
 
 ...
 
-=head1 ATTRIBUTES
-
-=over 4
-
-=item C<< canonical_media_type >>
-
-=item C<< file_extensions >>
-
-=back
-
 =head1 METHODS
 
 =over 4
@@ -35,13 +25,13 @@ This document describes AtteanX::Serializer::SPARQLXML version 0.034
 use v5.14;
 use warnings;
 
-package AtteanX::Serializer::SPARQLXML 0.034 {
+package AtteanX::Serializer::SPARQLXML 0.004 {
 	use Moo;
 	use Types::Standard qw(Str ArrayRef);
-	use Encode qw(encode encode_utf8);
+	use Encode qw(encode);
 	use Scalar::Util qw(blessed);
 	use Attean::ListIterator;
-	use List::Util qw(any);
+	use List::MoreUtils qw(any);
 	use namespace::clean;
 
 	has 'canonical_media_type' => (is => 'ro', isa => Str, init_arg => undef, default => 'application/sparql-results+xml');
@@ -55,14 +45,6 @@ Returns a list of media types that identify the format produced by this serializ
 	sub media_types {
 		return [qw(application/sparql-results+xml)];
 	}
-
-=item C<< file_extensions >>
-
-Returns a list of file extensions associated with the serialized format.
-
-=cut
-
-	sub file_extensions { return [qw(srx xml)] };
 	
 =item C<< serialize_iter_to_io( $fh, $iterator ) >>
 
@@ -80,9 +62,8 @@ L<IO::Handle> object C<< $fh >>.
 <sparql xmlns="http://www.w3.org/2005/sparql-results#">
 <head>
 END
-		my @vars	= @{ $iter->variables };
 		if ($iter->does('Attean::API::ResultIterator')) {
-			foreach my $v (@vars) {
+			foreach my $v (@{ $iter->variables }) {
 				print $fh qq(\t<variable name="$v"/>\n);
 			}
 		}
@@ -94,7 +75,7 @@ END
 		
 		while (my $t = $iter->next()) {
 			print $fh "\t\t<result>\n";
-			foreach my $name (@vars) {
+			foreach my $name ($t->variables) {
 				my $term	= $t->value($name);
 				if (blessed($term)) {
 					if ($term->does('Attean::API::IRI')) {
@@ -102,14 +83,12 @@ END
 						$label	=~ s/&/&amp;/g;
 						$label	=~ s/</&lt;/g;
 						$label	=~ s/"/&quot;/g;
-						$label	= encode_utf8($label);
 						print $fh qq(\t\t\t<binding name="${name}"><uri>${label}</uri></binding>\n);
 					} elsif ($term->does('Attean::API::Literal')) {
 						my $label	= $term->value;
 						$label	=~ s/&/&amp;/g;
 						$label	=~ s/</&lt;/g;
 						$label	=~ s/"/&quot;/g;
-						$label	= encode_utf8($label);
 						if (my $lang = $term->language) {
 							$label	= qq(<literal xml:lang="${lang}">${label}</literal>);
 						} elsif (my $dt = $term->datatype) {
@@ -123,10 +102,9 @@ END
 						$label	=~ s/&/&amp;/g;
 						$label	=~ s/</&lt;/g;
 						$label	=~ s/"/&quot;/g;
-						$label	= encode_utf8($label);
 						print $fh qq(\t\t\t<binding name="${name}"><bnode>${label}</bnode></binding>\n);
 					} else {
-						die "Term object has an unrecognized type: " . ref($term);
+						die;
 					}
 				}
 			}
@@ -137,7 +115,7 @@ END
 		return;
 	}
 	
-=item C<< serialize_iter_to_bytes( $iterator ) >>
+=item C<< serialize_iter_to_bytes( $fh, $iterator ) >>
 
 Serializes the L<Attean::API::Binding> objects from C<< $iterator >>
 and returns the serialization as a UTF-8 encoded byte string.
@@ -151,6 +129,8 @@ and returns the serialization as a UTF-8 encoded byte string.
 		open(my $fh, '>', \$data);
 		$self->serialize_iter_to_io($fh, $iter);
 		close($fh);
+		use Devel::Peek;
+		Dump($data);
 		return $data;
 	}
 
@@ -174,7 +154,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2014--2022 Gregory Todd Williams. This
+Copyright (c) 2006-2012 Gregory Todd Williams. This
 program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
