@@ -4,7 +4,7 @@ AtteanX::Store::Memory - Simple in-memory RDF store
 
 =head1 VERSION
 
-This document describes AtteanX::Store::Memory version 0.003
+This document describes AtteanX::Store::Memory version 0.004
 
 =head1 SYNOPSIS
 
@@ -19,7 +19,7 @@ AtteanX::Store::Memory provides an in-memory quad-store.
 use v5.14;
 use warnings;
 
-package AtteanX::Store::Memory 0.001 {
+package AtteanX::Store::Memory 0.004 {
 use Moo;
 use Type::Tiny::Role;
 use Types::Standard qw(Int ArrayRef HashRef ConsumerOf InstanceOf);
@@ -28,10 +28,14 @@ use Set::Scalar;
 use Digest::SHA;
 use List::Util qw(first);
 use Scalar::Util qw(refaddr reftype blessed);
+use Math::Cartesian::Product;
 use namespace::clean;
 
 with 'Attean::API::MutableQuadStore';
 with 'Attean::API::QuadStore';
+with 'Attean::API::ETagCacheableQuadStore';
+with 'Attean::API::TimeCacheableQuadStore';
+with 'Attean::API::CostPlanner';
 
 my @pos_names	= Attean::API::Quad->variables;
 
@@ -56,6 +60,7 @@ has object => (is => 'ro', isa => HashRef[InstanceOf['Set::Scalar']], init_arg =
 has graph => (is => 'ro', isa => HashRef[InstanceOf['Set::Scalar']], init_arg => undef, default => sub { +{} });
 has graph_nodes	=> (is => 'rw', isa => HashRef[ConsumerOf['Attean::API::IRI']], init_arg => undef, default => sub { +{} });
 has hash		=> (is => 'rw', isa => InstanceOf['Digest::SHA'], default => sub { Digest::SHA->new });
+has mtime		=> (is => 'rw', isa => Int, default => sub { return time() });
 
 =item C<< size >>
 
@@ -75,6 +80,14 @@ predicate and objects. Any of the arguments may be undef to match any value.
 =cut
 
 sub get_quads {
+	my $self	= shift;
+	my @nodes	= map { ref($_) eq 'ARRAY' ? $_ : [$_] } @_;
+	my @iters;
+	cartesian { push(@iters, $self->_get_quads(@_)) } @nodes;
+	return Attean::IteratorSequence->new( iterators => \@iters, item_type => 'Attean::API::Quad' );
+}
+
+sub _get_quads {
 	my $self	= shift;
 	my @nodes	= @_;
 	my $bound	= 0;
@@ -184,6 +197,7 @@ sub add_quad {
 		$self->_size($self->_size + 1);
 		my $id	= scalar(@{ $self->statements });
 		$self->hash->add('+' . encode_utf8($st->as_string));
+		$self->mtime(time());
 		push( @{ $self->statements }, $st );
 		foreach my $pos (0 .. $#pos_names) {
 			my $name	= $pos_names[ $pos ];
@@ -222,6 +236,7 @@ sub remove_quad {
 		$self->_size( $self->_size - 1 );
 		my $id	= $self->_statement_id( $st->values );
 		$self->hash->add('-' . encode_utf8($st->as_string));
+		$self->mtime(time());
 		$self->statements->[ $id ]	= undef;
 		foreach my $pos (0 .. 3) {
 			my $name	= $pos_names[ $pos ];
@@ -247,6 +262,13 @@ Removes the specified C<$statement> from the underlying model.
 =cut
 
 sub remove_quads {
+	my $self	= shift;
+	my @nodes	= map { ref($_) eq 'ARRAY' ? $_ : [$_] } @_;
+	my @iters;
+	cartesian { $self->_remove_quads(@_) } @nodes;
+}
+
+sub _remove_quads {
 	my $self	= shift;
 	my $subj	= shift;
 	my $pred	= shift;
@@ -307,6 +329,14 @@ value.
 =cut
 
 sub count_quads {
+	my $self	= shift;
+	my @nodes	= map { ref($_) eq 'ARRAY' ? $_ : [$_] } @_;
+	my $count	= 0;
+	cartesian { $count += $self->_count_quads(@_) } @nodes;
+	return $count;
+}
+
+sub _count_quads {
 	my $self	= shift;
 	my @nodes	= @_[0..3];
 	my $bound	= 0;
@@ -374,6 +404,11 @@ sub etag_value_for_quads {
 	return $self->hash->b64digest;
 }
 
+sub mtime_for_quads {
+	my $self	= shift;
+	return $self->mtime;
+}
+
 sub _statement_id {
 	my $self	= shift;
 	my @nodes	= @_;
@@ -410,6 +445,23 @@ sub _statement_id {
 	}
 }
 
+sub plans_for_algebra {
+	my $self	= shift;
+	my $algebra	= shift;
+	return;
+}
+
+sub cost_for_plan {
+	my $self	= shift;
+	my $plan	= shift;
+	if ($plan->isa('Attean::Plan::Quad')) {
+		my @values	= @{ $plan->values };
+		my $count	= $self->count_quads(@values);
+		return $count;
+	}
+	return;
+}
+
 }
 
 1;
